@@ -214,6 +214,18 @@ class OAuthClientTestCase(BaseClientTestCase):
         response = self.oauth.get_profile_info(raw_token)
         self.assertEqual(response, None)
 
+    def test_request_with_user_token(self, requests, auth):
+        "Use token for request auth."
+        token = 'oauth_token=token&oauth_token_secret=secret'
+        self.oauth = self.oauth_client(self.provider, token=token)
+        self.oauth.request('get', 'http://example.com/')
+        self.assertTrue(auth.called)
+        args, kwargs = auth.call_args
+        self.assertEqual(kwargs['client_key'], self.provider.key)
+        self.assertEqual(kwargs['client_secret'], self.provider.secret)
+        self.assertEqual(kwargs['resource_owner_key'], 'token')
+        self.assertEqual(kwargs['resource_owner_secret'], 'secret')
+
 
 @patch('allaccess.clients.request')
 class OAuth2ClientTestCase(BaseClientTestCase):
@@ -223,7 +235,8 @@ class OAuth2ClientTestCase(BaseClientTestCase):
 
     def test_access_token_url(self, requests):
         "Get should be sent to provider's access_token_url."
-        request = self.factory.get('/callback/', {'code': 'code'})
+        request = self.factory.get('/callback/', {'code': 'code', 'state': 'foo'})
+        request.session = {self.oauth.session_key: 'foo'}
         self.oauth.get_access_token(request)
         self.assertTrue(requests.called)
         args, kwargs = requests.call_args
@@ -233,7 +246,8 @@ class OAuth2ClientTestCase(BaseClientTestCase):
 
     def test_access_token_parameters(self, requests):
         "Check parameters used when fetching access token."
-        request = self.factory.get('/callback/', {'code': 'code'})
+        request = self.factory.get('/callback/', {'code': 'code', 'state': 'foo'})
+        request.session = {self.oauth.session_key: 'foo'}
         self.oauth.get_access_token(request)
         self.assertTrue(requests.called)
         args, kwargs = requests.call_args
@@ -246,7 +260,8 @@ class OAuth2ClientTestCase(BaseClientTestCase):
 
     def test_access_token_custom_callback(self, requests):
         "Check parameters used with custom callback."
-        request = self.factory.get('/callback/', {'code': 'code'})
+        request = self.factory.get('/callback/', {'code': 'code', 'state': 'foo'})
+        request.session = {self.oauth.session_key: 'foo'}
         self.oauth.get_access_token(request, callback='/other/')
         self.assertTrue(requests.called)
         args, kwargs = requests.call_args
@@ -259,7 +274,8 @@ class OAuth2ClientTestCase(BaseClientTestCase):
 
     def test_access_token_no_code(self, requests):
         "Don't request token if no code was given to the callback."
-        request = self.factory.get('/callback/')
+        request = self.factory.get('/callback/', {'state': 'foo'})
+        request.session = {self.oauth.session_key: 'foo'}
         token = self.oauth.get_access_token(request)
         self.assertEqual(token, None)
         self.assertFalse(requests.called)
@@ -269,14 +285,16 @@ class OAuth2ClientTestCase(BaseClientTestCase):
         response = Mock()
         response.text = 'access_token=USER_ACESS_TOKEN'
         requests.return_value = response
-        request = self.factory.get('/callback/', {'code': 'code'})
+        request = self.factory.get('/callback/', {'code': 'code', 'state': 'foo'})
+        request.session = {self.oauth.session_key: 'foo'}
         token = self.oauth.get_access_token(request)
         self.assertEqual(token, 'access_token=USER_ACESS_TOKEN')
 
     def test_access_token_failure(self, requests):
         "Handle upstream server errors when fetching access token."
         requests.side_effect = RequestException('Server Down')
-        request = self.factory.get('/callback/', {'code': 'code'})
+        request = self.factory.get('/callback/', {'code': 'code', 'state': 'foo'})
+        request.session = {self.oauth.session_key: 'foo'}
         token = self.oauth.get_access_token(request)
         self.assertEqual(token, None)
 
@@ -332,3 +350,36 @@ class OAuth2ClientTestCase(BaseClientTestCase):
         token, secret = self.oauth.parse_raw_token(raw_token)
         self.assertEqual(token, None)
         self.assertEqual(secret, None)
+
+    def test_access_token_no_state_session(self, requests):
+        "Handle no state found in the session."
+        request = self.factory.get('/callback/', {'code': 'code', 'state': 'foo'})
+        request.session = {}
+        response = self.oauth.get_access_token(request)
+        self.assertEqual(response, None)
+        self.assertFalse(requests.called)
+
+    def test_access_token_no_state_provider(self, requests):
+        "Handle no state returned by the provider."
+        request = self.factory.get('/callback/', {'code': 'code'})
+        request.session = {self.oauth.session_key: 'foo'}
+        response = self.oauth.get_access_token(request)
+        self.assertEqual(response, None)
+        self.assertFalse(requests.called)
+
+    def test_access_token_state_incorrect(self, requests):
+        "Handle invalid state returned by the provider"
+        request = self.factory.get('/callback/', {'code': 'code', 'state': 'bar'})
+        request.session = {self.oauth.session_key: 'foo'}
+        response = self.oauth.get_access_token(request)
+        self.assertEqual(response, None)
+        self.assertFalse(requests.called)
+
+    def test_request_with_user_token(self, requests):
+        "Use token for request auth."
+        token = '{"access_token": "USER_ACESS_TOKEN"}'
+        self.oauth = self.oauth_client(self.provider, token=token)
+        self.oauth.request('get', 'http://example.com/')
+        self.assertTrue(requests.called)
+        args, kwargs = requests.call_args
+        self.assertEqual(kwargs['params']['access_token'], 'USER_ACESS_TOKEN')
